%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  plot ROMS-1D variables
%
% Patrick Marchesiello - June 2012
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all
close all

romstools_param

OUTDIR='OUT_CONTROL/';

dstr=220; dend=225;
if (lat<0),
 dstr=dstr-180; dend=dend-180;
end
zmax=-100;
%=============================================
fname=[OUTDIR,'t01.dat'];
var=load(fname);
temp=var';
fname=[OUTDIR,'days.dat'];
j=load(fname);
NT=length(j);
fname=[OUTDIR,'z.dat'];
z=load(fname);
N=length(z);
fname=[OUTDIR,'zbl.dat'];
zbl=load(fname);
windowSize = 8;
mzbl=filter(ones(1,windowSize)/windowSize,1,zbl);
fname=[OUTDIR,'t02.dat'];
var=load(fname);
salt=var';
fname=[OUTDIR,'t03.dat'];
var=load(fname);
no3=var';
fname=[OUTDIR,'t05.dat'];
var=load(fname);
chla=var';
fname=[OUTDIR,'akt.dat'];
akt=load(fname);
akt=akt';
%
tstr=min(find(j>dstr));
tend=min(find(j>dend));
if isempty(tend), tend=length(j); end;

[z_r,z_w,Hz]=get_grid(N,theta_s,hmax);
z_r2d=repmat(z_r,NT,1)';
z_w2d=repmat(z_w,NT,1)';
[rho,bvf]=rho_eos(temp,salt,z_r2d,z_w2d,9.8,1025);

%
% seasonal cycle of T
%
figure(3)
%cmin=15; cmax=30; cint=0.25;
K=min(find(z>-100))-1;
cmin=min(min(temp(K:end,:))); 
cmax=max(max(temp(K:end,:))); cint=0.25;
contourf(j(4:8:end),z,temp(:,4:8:end),[cmin:cint:cmax]);
colorbar
hold on
h=plot(j,mzbl,'k');
hold off
set(h,'LineWidth',2)
xlabel('Time [Days]')
ylabel('Depth [Meters]')
title('T [^{\circ} C]')
caxis([cmin cmax]);
axis([min(j) max(j) -100 0]);
print -dpdf temp_sea.pdf

%
% Summer T
%
figure(4)
%cmin=18; cmax=30; cint=0.05;
K=min(find(z>zmax))-1;
cmin=min(min(temp(K:end,tstr:tend))); 
cmax=max(max(temp(K:end,tstr:tend))); cint=0.05;
contourf(j,z,temp,[cmin:cint:cmax]);
colorbar
hold on
h=plot(j,zbl,'k');
hold off
set(h,'LineWidth',2)
xlabel('Time [Days]')
ylabel('Depth [Meters]')
title('T [^{\circ} C]')
caxis([cmin cmax]);
axis([dstr dend zmax 0]);
print -dpdf temp.pdf

%
% Summer S
%
figure(5)
K=min(find(z>zmax))-1;
cmin=min(min(salt(K:end,tstr:tend))); 
cmax=max(max(salt(K:end,tstr:tend))); cint=0.005;
contourf(j,z,salt,[cmin:cint:cmax]);
colorbar
hold on
plot(j,zbl,'k');
hold off
set(h,'LineWidth',2)
xlabel('Time [Days]')
ylabel('Depth [Meters]')
title('S [psu]')
caxis([cmin cmax]);
axis([dstr dend zmax 0]);
print -dpdf salt.pdf

%
% Summer rho
%
figure(6)
K=min(find(z>zmax))-1;
cmin=min(min(rho(K:end,tstr:tend))); 
cmax=max(max(rho(K:end,tstr:tend))); 
pcolor(j,z,rho); shading flat;
colorbar
hold on
h=plot(j,zbl,'k');
hold off
set(h,'LineWidth',2)
xlabel('Time [Days]')
ylabel('Depth [Meters]')
title('RHO [kg/m^3]')
caxis([cmin cmax]);
axis([dstr dend zmax 0]);

%
% Summer bvf
%
figure(7)
K=min(find(z>zmax))-1;
cmin=min(min(bvf(K:end,tstr:tend))); 
cmax=max(max(bvf(K:end,tstr:tend)));
%cmin=min(cmin,-cmax);cmax=max(cmax,-cmin);
pcolor(j,z_w,bvf); shading flat
colorbar
hold on
h=plot(j,zbl,'k');
hold off
set(h,'LineWidth',2)
xlabel('Time [Days]')
ylabel('Depth [Meters]')
title('BVF [1/s^2]')
caxis([cmin cmax]);
axis([dstr dend zmax 0]);
print -dpdf bvf.pdf

%
% Summer Akt
%
figure(8)
cmin=0; cmax=4; cint=1;
%contourf(j,z,akt,[cmin:cint:cmax]);
pcolor(j,z,log10(akt))
shading flat
colorbar
hold on
h=plot(j,zbl,'k');
hold off
set(h,'LineWidth',2)
xlabel('Time [Days]')
ylabel('Depth [Meters]')
title('LOG(AKt) [cm^2/s]')
caxis([cmin cmax]);
axis([dstr dend zmax 0]);
print -dpdf akt.pdf

%
% Summer NO3
%
figure(9)
%cmin=18; cmax=30; cint=0.05;
K=min(find(z>zmax))-1;
var=log10(no3);
cmin=min(min(var(K:end,tstr:tend))); 
cmax=max(max(var(K:end,tstr:tend))); cint=0.05;
contourf(j,z,var,[cmin:cint:cmax]);
%pcolor(j,z,var); shading flat
colorbar
hold on
h=plot(j,zbl,'k');
hold off
set(h,'LineWidth',2)
xlabel('Time [Days]')
ylabel('Depth [Meters]')
title('LOG(NO3) [mMol N m-3]')
caxis([cmin cmax]);
axis([dstr dend zmax 0]);
print -dpdf no3.pdf
%

% Summer Chla
%
figure(10)
%cmin=18; cmax=30; cint=0.05;
K=min(find(z>zmax))-1;
var=log10(chla);
cmin=min(min(var(K:end,tstr:tend))); 
cmax=max(max(var(K:end,tstr:tend))); cint=0.01;
contourf(j,z,var,[cmin:cint:cmax]);
%pcolor(j,z,var); shading flat
colorbar
hold on
h=plot(j,zbl,'k');
hold off
set(h,'LineWidth',2)
xlabel('Time [Days]')
ylabel('Depth [Meters]')
title('LOG(CHLa) [mg Chl m-3]')
caxis([cmin cmax]);
axis([dstr dend zmax 0]);
print -dpdf chl.pdf
